from os import path
import autofit as af
import autofit.non_linear.paths
import autogalaxy as ag
from autogalaxy.hyper import hyper_data as hd
import pytest
from autogalaxy.fit.fit import FitImaging
from autogalaxy.mock import mock


class MockPhase:
    def __init__(self):
        self.name = "name"
        self.paths = autofit.non_linear.paths.Paths(
            name=self.name, path_prefix="phase_path", tag=""
        )
        self.search = mock.MockSearch(paths=self.paths)
        self.model = af.ModelMapper()

    def save_dataset(self, dataset):
        pass

    # noinspection PyUnusedLocal,PyMethodMayBeStatic
    def run(self, *args, **kwargs):
        return mock.MockResult()


class TestModelFixing:
    def test__defaults_both(self):
        # noinspection PyTypeChecker
        phase = ag.ModelFixingHyperPhase(
            phase=MockPhase(),
            hyper_name="test",
            hyper_search=mock.MockSearch(),
            model_classes=(hd.HyperImageSky, hd.HyperBackgroundNoise),
        )

        instance = af.ModelInstance()
        instance.hyper_image_sky = ag.hyper_data.HyperImageSky()
        instance.hyper_background_noise = ag.hyper_data.HyperBackgroundNoise()

        mapper = phase.make_model(instance=instance)

        assert isinstance(mapper.hyper_image_sky, af.PriorModel)
        assert isinstance(mapper.hyper_background_noise, af.PriorModel)

        assert mapper.hyper_image_sky.cls == ag.hyper_data.HyperImageSky
        assert mapper.hyper_background_noise.cls == ag.hyper_data.HyperBackgroundNoise

    def test__make_pixelization_model(self):
        instance = af.ModelInstance()
        mapper = af.ModelMapper()

        mapper.galaxy = ag.GalaxyModel(
            redshift=ag.Redshift,
            pixelization=ag.pix.Rectangular,
            regularization=ag.reg.Constant,
        )
        mapper.source_galaxy = ag.GalaxyModel(
            redshift=ag.Redshift, light=ag.lp.EllipticalLightProfile
        )

        assert mapper.prior_count == 10

        instance.galaxy = ag.Galaxy(
            pixelization=ag.pix.Rectangular(),
            regularization=ag.reg.Constant(),
            redshift=1.0,
        )
        instance.source_galaxy = ag.Galaxy(
            redshift=1.0, light=ag.lp.EllipticalLightProfile()
        )

        # noinspection PyTypeChecker
        phase = ag.ModelFixingHyperPhase(
            phase=MockPhase(),
            hyper_name="mock_phase",
            hyper_search=mock.MockSearch(),
            model_classes=(ag.pix.Pixelization, ag.reg.Regularization),
        )

        mapper = mapper.copy_with_fixed_priors(instance, phase.model_classes)

        assert mapper.prior_count == 3
        assert mapper.galaxy.redshift == 1.0
        assert mapper.source_galaxy.light.elliptical_comps[0] == 0.0


@pytest.fixture(name="hyper_combined")
def make_combined():
    normal_phase = MockPhase()

    # noinspection PyUnusedLocal
    def run_hyper(*args, **kwargs):
        return mock.MockResult()

    hyper_galaxy_phase = ag.HyperGalaxyPhase(
        phase=normal_phase,
        hyper_search=mock.MockSearch(),
        include_sky_background=False,
        include_noise_background=False,
    )
    inversion_phase = ag.InversionPhase(
        phase=normal_phase,
        hyper_search=mock.MockSearch(),
        model_classes=(ag.pix.Pixelization, ag.reg.Regularization),
    )

    # noinspection PyTypeChecker
    hyper_combined = ag.CombinedHyperPhase(
        phase=normal_phase,
        hyper_phases=(hyper_galaxy_phase, inversion_phase),
        hyper_search=mock.MockSearch(),
    )

    for phase in hyper_combined.hyper_phases:
        phase.run_hyper = run_hyper

    return hyper_combined


class TestHyperAPI:
    def test_combined_result(self, hyper_combined):
        result = hyper_combined.run(
            dataset=None, mask=None, results=af.ResultsCollection()
        )

        assert hasattr(result, "hyper_galaxy")
        assert isinstance(result.hyper_galaxy, mock.MockResult)

        assert hasattr(result, "inversion")
        assert isinstance(result.inversion, mock.MockResult)

        assert hasattr(result, "hyper_combined")
        assert isinstance(result.hyper_combined, mock.MockResult)

    def test_combine_models(self, hyper_combined):
        result = mock.MockResult()
        hyper_galaxy_result = mock.MockResult()
        inversion_result = mock.MockResult()

        hyper_galaxy_result.model = af.ModelMapper()
        inversion_result.model = af.ModelMapper()

        hyper_galaxy_result.model.hyper_galaxy = ag.HyperGalaxy
        hyper_galaxy_result.model.pixelization = ag.pix.Pixelization()
        inversion_result.model.pixelization = ag.pix.Pixelization
        inversion_result.model.hyper_galaxy = ag.HyperGalaxy()

        result.hyper_galaxy = hyper_galaxy_result
        result.inversion = inversion_result

        model = hyper_combined.combine_models(result)

        assert isinstance(model.hyper_galaxy, af.PriorModel)
        assert isinstance(model.pixelization, af.PriorModel)

        assert model.hyper_galaxy.cls == ag.HyperGalaxy
        assert model.pixelization.cls == ag.pix.Pixelization

    def test_instantiation(self, hyper_combined):
        assert len(hyper_combined.hyper_phases) == 2

        galaxy_phase = hyper_combined.hyper_phases[0]
        pixelization_phase = hyper_combined.hyper_phases[1]

        assert galaxy_phase.hyper_name == "hyper_galaxy"
        assert isinstance(galaxy_phase, ag.HyperGalaxyPhase)

        assert pixelization_phase.hyper_name == "inversion"
        assert isinstance(pixelization_phase, ag.InversionPhase)

    def test_hyper_result(self, imaging_7x7, mask_7x7):
        normal_phase = MockPhase()

        # noinspection PyTypeChecker
        phase = ag.HyperGalaxyPhase(
            phase=normal_phase,
            hyper_search=mock.MockSearch(),
            include_sky_background=False,
            include_noise_background=False,
        )

        # noinspection PyUnusedLocal
        def run_hyper(*args, **kwargs):
            return mock.MockResult()

        phase.run_hyper = run_hyper

        result = phase.run(dataset=imaging_7x7, results=af.ResultsCollection())

        assert hasattr(result, "hyper_galaxy")
        assert isinstance(result.hyper_galaxy, mock.MockResult)

    def test__paths(self):

        galaxy = ag.Galaxy(
            pixelization=ag.pix.Rectangular(),
            regularization=ag.reg.Constant(),
            redshift=1.0,
        )

        phase = ag.PhaseImaging(
            galaxies=dict(galaxy=galaxy),
            search=af.DynestyStatic(n_live_points=1, name="test_phase"),
            settings=ag.SettingsPhaseImaging(
                settings_masked_imaging=ag.SettingsMaskedImaging(bin_up_factor=2)
            ),
        )

        phase_extended = phase.extend_with_inversion_phase(
            hyper_search=af.DynestyStatic(n_live_points=1)
        )

        phase_extended.paths.path_prefix = "prefix"
        hyper_phase = phase_extended.make_hyper_phase()
        hyper_phase.modify_search_paths()

        assert (
            path.join(
                "output",
                "prefix",
                "test_phase",
                "inversion__settings__imaging[grid_sub_2_inv_sub_2__bin_2]__pix[no_border]__inv[mat]",
                "dynesty_static[nlive_1",
            )
            in hyper_phase.paths.output_path
        )

        phase_extended = phase.extend_with_multiple_hyper_phases(
            setup_hyper=ag.SetupHyper(
                hyper_galaxies=True,
                inversion_search=af.DynestyStatic(n_live_points=1),
                hyper_galaxies_search=af.DynestyStatic(n_live_points=2),
                hyper_combined_search=af.DynestyStatic(n_live_points=3),
            ),
            include_inversion=True,
        )

        inversion_phase = phase_extended.hyper_phases[0].make_hyper_phase()
        inversion_phase.modify_search_paths()

        assert (
            path.join(
                "test_phase",
                "inversion__settings__imaging[grid_sub_2_inv_sub_2__bin_2]__pix[no_border]__inv[mat]",
                "dynesty_static[nlive_1",
            )
            in inversion_phase.paths.output_path
        )

        hyper_galaxy_phase = phase_extended.hyper_phases[1].make_hyper_phase()
        hyper_galaxy_phase.modify_search_paths()

        assert (
            path.join(
                "test_phase",
                "hyper_galaxy__settings__imaging[grid_sub_2_inv_sub_2__bin_2]__pix[no_border]__inv[mat]",
                "dynesty_static[nlive_2",
            )
            in hyper_galaxy_phase.paths.output_path
        )

        hyper_combined_phase = phase_extended.make_hyper_phase()
        hyper_combined_phase.modify_search_paths()

        assert (
            path.join(
                "test_phase",
                "hyper_combined__settings__imaging[grid_sub_2_inv_sub_2__bin_2]__pix[no_border]__inv[mat]",
                "dynesty_static[nlive_3",
            )
            in hyper_combined_phase.paths.output_path
        )


class TestHyperGalaxyPhase:
    def test__likelihood_function_is_same_as_normal_phase_likelihood_function(
        self, imaging_7x7, mask_7x7
    ):

        hyper_image_sky = ag.hyper_data.HyperImageSky(sky_scale=1.0)
        hyper_background_noise = ag.hyper_data.HyperBackgroundNoise(noise_scale=1.0)

        galaxy = ag.Galaxy(redshift=0.5, light=ag.lp.EllipticalSersic(intensity=0.1))

        phase_imaging_7x7 = ag.PhaseImaging(
            galaxies=dict(galaxy=galaxy),
            hyper_image_sky=hyper_image_sky,
            hyper_background_noise=hyper_background_noise,
            settings=ag.SettingsPhaseImaging(
                settings_masked_imaging=ag.SettingsMaskedImaging(sub_size=2)
            ),
            search=mock.MockSearch(name="test_phase"),
        )

        analysis = phase_imaging_7x7.make_analysis(
            dataset=imaging_7x7, mask=mask_7x7, results=mock.MockResults()
        )
        instance = phase_imaging_7x7.model.instance_from_unit_vector([])

        assert analysis.masked_dataset.mask.sub_size == 2

        masked_imaging = ag.MaskedImaging(
            imaging=imaging_7x7,
            mask=mask_7x7,
            settings=ag.SettingsMaskedImaging(sub_size=2),
        )
        plane = analysis.plane_for_instance(instance=instance)
        fit = FitImaging(
            masked_imaging=masked_imaging,
            plane=plane,
            hyper_image_sky=hyper_image_sky,
            hyper_background_noise=hyper_background_noise,
        )

        phase_imaging_7x7_hyper = phase_imaging_7x7.extend_with_multiple_hyper_phases(
            setup_hyper=ag.SetupHyper(
                hyper_galaxies=True, hyper_galaxies_search=mock.MockSearch()
            )
        )

        instance = phase_imaging_7x7_hyper.model.instance_from_unit_vector([])

        instance.hyper_galaxy = ag.HyperGalaxy(noise_factor=0.0)

        analysis = phase_imaging_7x7_hyper.hyper_phases[0].Analysis(
            masked_imaging=masked_imaging,
            hyper_model_image=fit.model_image,
            hyper_galaxy_image=fit.model_image,
        )

        fit_hyper = analysis.fit_for_hyper_galaxy(
            hyper_galaxy=ag.HyperGalaxy(noise_factor=0.0),
            hyper_image_sky=hyper_image_sky,
            hyper_background_noise=hyper_background_noise,
        )

        assert fit_hyper.log_likelihood == fit.log_likelihood

        fit_hyper = analysis.fit_for_hyper_galaxy(
            hyper_galaxy=ag.HyperGalaxy(noise_factor=1.0),
            hyper_image_sky=hyper_image_sky,
            hyper_background_noise=hyper_background_noise,
        )

        assert fit_hyper.log_likelihood != fit.log_likelihood

        instance.hyper_galaxy = ag.HyperGalaxy(noise_factor=0.0)

        log_likelihood = analysis.log_likelihood_function(instance=instance)

        assert log_likelihood == fit.log_likelihood



import os
import numpy as np
from autoconf import conf
import autofit as af
import autogalaxy as ag
from autofit.non_linear.mock_search import MockSearch
from test_autogalaxy.simulators.interferometer import instrument_util


def run(module, test_name=None, search=af.DynestyStatic(), config_folder="config"):
    test_name = test_name or module.test_name
    test_path = "{}/../../".format(os.path.dirname(os.path.realpath(__file__)))
    output_path = f"{test_path}/output/interferometer/"
    config_path = test_path + config_folder
    conf.instance.push(new_path, output_path=output_path)

    interferometer = instrument_util.load_test_interferometer(
        data_name=module.dataset_name, instrument=module.instrument
    )

    pixel_scales = instrument_util.pixel_scale_from_instrument(
        instrument=module.instrument
    )
    grid = instrument_util.grid_from_instrument(instrument=module.instrument)

    real_space_mask = ag.Mask2D.circular(
        shape_2d=grid.shape_2d, pixel_scales=pixel_scales, radius=2.0
    )

    visibilities_mask = np.full(
        fill_value=False, shape=interferometer.visibilities.shape
    )

    module.make_pipeline(
        name=test_name,
        path_prefix=[module.test_type, test_name],
        search=search,
        real_space_mask=real_space_mask,
    ).run(dataset=interferometer, mask=visibilities_mask)


def run_a_mock(module):
    # noinspection PyTypeChecker
    run(
        module,
        test_name=f"{module.test_name}_mock",
        search=MockSearch,
        config_folder="config_mock",
    )


def run_with_multi_nest(module):
    # noinspection PyTypeChecker
    run(
        module,
        test_name=f"{module.test_name}_nest",
        search=af.DynestyStatic(),
        config_folder="config_mock",
    )



from os import path

import autogalaxy as ag
import autogalaxy.plot as aplt
import pytest
from autoconf import conf

directory = path.dirname(path.realpath(__file__))


@pytest.fixture(name="plot_path")
def make_plane_plotter_setup():
    return path.join(
        "{}".format(path.dirname(path.realpath(__file__))), "files", "plots", "plane"
    )


@pytest.fixture(autouse=True)
def set_config_path():
    conf.instance = conf.Config(
        path.join(directory, "files", "plotter"), path.join(directory, "output")
    )


def test__all_individual_plotters__output_file_with_default_name(
    plane_7x7, sub_grid_7x7, mask_7x7, positions_7x7, include_all, plot_path, plot_patch
):

    aplt.Plane.image(
        plane=plane_7x7,
        grid=sub_grid_7x7,
        positions=positions_7x7,
        include=include_all,
        plotter=aplt.Plotter(output=aplt.Output(plot_path, format="png")),
    )

    assert path.join(plot_path, "image.png") in plot_patch.paths

    aplt.Plane.plane_image(
        plane=plane_7x7,
        grid=sub_grid_7x7,
        positions=positions_7x7,
        include=include_all,
        plotter=aplt.Plotter(output=aplt.Output(plot_path, format="png")),
    )

    assert path.join(plot_path, "plane_image.png") in plot_patch.paths

    aplt.Plane.convergence(
        plane=plane_7x7,
        grid=sub_grid_7x7,
        include=include_all,
        plotter=aplt.Plotter(output=aplt.Output(plot_path, format="png")),
    )

    assert path.join(plot_path, "convergence.png") in plot_patch.paths

    aplt.Plane.potential(
        plane=plane_7x7,
        grid=sub_grid_7x7,
        include=include_all,
        plotter=aplt.Plotter(output=aplt.Output(plot_path, format="png")),
    )

    assert path.join(plot_path, "potential.png") in plot_patch.paths

    aplt.Plane.deflections_y(
        plane=plane_7x7,
        grid=sub_grid_7x7,
        include=include_all,
        plotter=aplt.Plotter(output=aplt.Output(plot_path, format="png")),
    )

    assert path.join(plot_path, "deflections_y.png") in plot_patch.paths

    aplt.Plane.deflections_x(
        plane=plane_7x7,
        grid=sub_grid_7x7,
        include=include_all,
        plotter=aplt.Plotter(output=aplt.Output(plot_path, format="png")),
    )

    assert path.join(plot_path, "deflections_x.png") in plot_patch.paths

    aplt.Plane.magnification(
        plane=plane_7x7,
        grid=sub_grid_7x7,
        include=include_all,
        plotter=aplt.Plotter(output=aplt.Output(plot_path, format="png")),
    )

    assert path.join(plot_path, "magnification.png") in plot_patch.paths

    aplt.Plane.plane_grid(
        plane=plane_7x7,
        grid=sub_grid_7x7,
        include=include_all,
        plotter=aplt.Plotter(output=aplt.Output(plot_path, format="png")),
    )

    assert path.join(plot_path, "plane_grid.png") in plot_patch.paths

    plane_7x7.galaxies[0].hyper_galaxy = ag.HyperGalaxy()
    plane_7x7.galaxies[0].hyper_model_image = ag.Array.ones(
        shape_2d=(7, 7), pixel_scales=0.1
    )
    plane_7x7.galaxies[0].hyper_galaxy_image = ag.Array.ones(
        shape_2d=(7, 7), pixel_scales=0.1
    )

    aplt.Plane.contribution_map(
        plane=plane_7x7,
        mask=mask_7x7,
        positions=positions_7x7,
        include=include_all,
        plotter=aplt.Plotter(output=aplt.Output(plot_path, format="png")),
    )

    assert path.join(plot_path, "contribution_map.png") in plot_patch.paths



import copy
from os import path
import autofit as af
import numpy as np
from autoarray.fit import fit as aa_fit
from autogalaxy.fit import fit
from autogalaxy.galaxy import galaxy as g
from autogalaxy.hyper import hyper_data as hd
from autogalaxy.pipeline import visualizer

from .hyper_phase import HyperPhase


class Analysis(af.Analysis):
    def __init__(self, masked_imaging, hyper_model_image, hyper_galaxy_image):
        """
        An analysis to fit the noise for a single galaxy image.
        Parameters
        ----------
        masked_imaging: LensData
            lens dataset, including an image and noise
        hyper_model_image: np.ndarray
            An image produce of the overall system by a model
        hyper_galaxy_image: np.ndarray
            The contribution of one galaxy to the model image
        """

        super().__init__()

        self.masked_imaging = masked_imaging

        self.visualizer = visualizer.HyperGalaxyVisualizer()

        self.hyper_model_image = hyper_model_image
        self.hyper_galaxy_image = hyper_galaxy_image

    def visualize(self, paths, instance, during_analysis):

        if self.visualizer.plot_hyper_galaxy_subplot:
            hyper_image_sky = self.hyper_image_sky_for_instance(instance=instance)

            hyper_background_noise = self.hyper_background_noise_for_instance(
                instance=instance
            )

            contribution_map = instance.hyper_galaxy.contribution_map_from_hyper_images(
                hyper_model_image=self.hyper_model_image,
                hyper_galaxy_image=self.hyper_galaxy_image,
            )

            fit_normal = aa_fit.FitImaging(
                masked_imaging=self.masked_imaging,
                model_image=self.hyper_model_image,
                use_mask_in_fit=False,
            )

            fit_hyper = self.fit_for_hyper_galaxy(
                hyper_galaxy=instance.hyper_galaxy,
                hyper_image_sky=hyper_image_sky,
                hyper_background_noise=hyper_background_noise,
            )

            self.visualizer.visualize_hyper_galaxy(
                paths=paths,
                fit=fit_normal,
                hyper_fit=fit_hyper,
                galaxy_image=self.hyper_galaxy_image,
                contribution_map_in=contribution_map,
            )

    def log_likelihood_function(self, instance):
        """
        Fit the model image to the real image by scaling the hyper_galaxies noise.
        Parameters
        ----------
        instance: ModelInstance
            A model instance with a hyper_galaxies galaxy property
        Returns
        -------
        fit: float
        """

        hyper_image_sky = self.hyper_image_sky_for_instance(instance=instance)

        hyper_background_noise = self.hyper_background_noise_for_instance(
            instance=instance
        )

        fit = self.fit_for_hyper_galaxy(
            hyper_galaxy=instance.hyper_galaxy,
            hyper_image_sky=hyper_image_sky,
            hyper_background_noise=hyper_background_noise,
        )

        return fit.figure_of_merit

    @staticmethod
    def hyper_image_sky_for_instance(instance):
        if hasattr(instance, "hyper_image_sky"):
            return instance.hyper_image_sky

    @staticmethod
    def hyper_background_noise_for_instance(instance):
        if hasattr(instance, "hyper_background_noise"):
            return instance.hyper_background_noise

    def fit_for_hyper_galaxy(
        self, hyper_galaxy, hyper_image_sky, hyper_background_noise
    ):

        image = fit.hyper_image_from_image_and_hyper_image_sky(
            image=self.masked_imaging.image, hyper_image_sky=hyper_image_sky
        )

        if hyper_background_noise is not None:
            noise_map = hyper_background_noise.hyper_noise_map_from_noise_map(
                noise_map=self.masked_imaging.noise_map
            )
        else:
            noise_map = self.masked_imaging.noise_map

        hyper_noise_map = hyper_galaxy.hyper_noise_map_from_hyper_images_and_noise_map(
            hyper_model_image=self.hyper_model_image,
            hyper_galaxy_image=self.hyper_galaxy_image,
            noise_map=self.masked_imaging.noise_map,
        )

        noise_map = noise_map + hyper_noise_map

        masked_imaging = self.masked_imaging.modify_image_and_noise_map(
            image=image, noise_map=noise_map
        )

        return aa_fit.FitImaging(
            masked_imaging=masked_imaging,
            model_image=self.hyper_model_image,
            use_mask_in_fit=False,
        )

    @classmethod
    def describe(cls, instance):
        return "Running hyper_galaxies galaxy fit for HyperGalaxy:\n{}".format(
            instance.hyper_galaxy
        )


class HyperGalaxyPhase(HyperPhase):
    Analysis = Analysis

    def __init__(
        self,
        phase,
        hyper_search,
        include_sky_background,
        include_noise_background,
        hyper_galaxy_names=None,
    ):

        super().__init__(
            phase=phase, hyper_search=hyper_search, hyper_name="hyper_galaxy"
        )
        self.include_sky_background = include_sky_background
        self.include_noise_background = include_noise_background
        self.hyper_galaxy_names = hyper_galaxy_names

    def run_hyper(
        self, dataset, results: af.ResultsCollection, info=None, pickle_files=None
    ):
        """
        Run a fit for each galaxy from the previous phase.
        Parameters
        ----------
        dataset: LensData
        results: ResultsCollection
            Results from all previous phases
        Returns
        -------
        results: HyperGalaxyResults
            A collection of results, with one item per a galaxy
        """

        self.results = results

        phase = self.make_hyper_phase()
        phase.modify_search_paths()

        masked_imaging = results.last.masked_dataset

        hyper_result = copy.deepcopy(results.last)
        hyper_result.model = hyper_result.model.copy_with_fixed_priors(
            hyper_result.instance
        )

        hyper_result.analysis.hyper_model_image = results.last.hyper_model_image
        hyper_result.analysis.hyper_galaxy_image_path_dict = (
            results.last.hyper_galaxy_image_path_dict
        )

        for path_galaxy, galaxy in results.last.path_galaxy_tuples:

            if self.hyper_galaxy_names is not None:
                if path_galaxy[-1] in self.hyper_galaxy_names:

                    model = copy.deepcopy(phase.model)
                    search = copy.deepcopy(phase.search)
                    search.paths.tag = path.join(search.paths.tag, path_galaxy[-1])

                    # TODO : This is a HACK :O

                    model.galaxies = []

                    model.hyper_galaxy = g.HyperGalaxy

                    if self.include_sky_background:
                        model.hyper_image_sky = hd.HyperImageSky

                    if self.include_noise_background:
                        model.hyper_background_noise = hd.HyperBackgroundNoise

                    # If arrays is all zeros, galaxy did not have image in previous phase and hyper phase is omitted.
                    if not np.all(
                        hyper_result.analysis.hyper_galaxy_image_path_dict[path_galaxy]
                        == 0
                    ):
                        hyper_model_image = hyper_result.analysis.hyper_model_image

                        analysis = self.Analysis(
                            masked_imaging=masked_imaging,
                            hyper_model_image=hyper_model_image,
                            hyper_galaxy_image=hyper_result.analysis.hyper_galaxy_image_path_dict[
                                path_galaxy
                            ],
                        )

                        result = search.fit(
                            model=model,
                            analysis=analysis,
                            info=info,
                            pickle_files=pickle_files,
                        )

                        def transfer_field(name):
                            if hasattr(result._instance, name):
                                setattr(
                                    hyper_result._instance.object_for_path(path_galaxy),
                                    name,
                                    getattr(result._instance, name),
                                )
                                setattr(
                                    hyper_result.model.object_for_path(path_galaxy),
                                    name,
                                    getattr(result.model, name),
                                )

                        transfer_field("hyper_galaxy")

                        hyper_result._instance.hyper_image_sky = getattr(
                            result._instance, "hyper_image_sky"
                        )
                        hyper_result.model.hyper_image_sky = getattr(
                            result.model, "hyper_image_sky"
                        )

                        hyper_result._instance.hyper_background_noise = getattr(
                            result._instance, "hyper_background_noise"
                        )
                        hyper_result.model.hyper_background_noise = getattr(
                            result.model, "hyper_background_noise"
                        )

        return hyper_result



import autofit as af
import autogalaxy as ag
from test_autogalaxy.integration.tests.interferometer import runner

test_type = "galaxy_x1"
test_name = "galaxy_bulge__hyper_bg"
data_name = "galaxy_x1__dev_vaucouleurs"
instrument = "sma"


def make_pipeline(name, path_prefix, real_space_mask, search=af.DynestyStatic()):

    phase1 = ag.PhaseInterferometer(
        name="phase_1",
        path_prefix=path_prefix,
        galaxies=dict(
            galaxy=ag.GalaxyModel(redshift=0.5, bulge=ag.lp.EllipticalSersic)
        ),
        real_space_mask=real_space_mask,
        search=search,
    )

    phase1.search.const_efficiency_mode = True
    phase1.search.n_live_points = 40
    phase1.search.facc = 0.8

    phase1 = phase1.extend_with_multiple_hyper_phases(
        hyper_galaxies_search=True,
        include_background_sky=True,
        include_background_noise=True,
    )

    phase2 = ag.PhaseInterferometer(
        name="phase_2",
        path_prefix=path_prefix,
        galaxies=dict(
            galaxy=ag.GalaxyModel(
                redshift=0.5,
                bulge=phase1.result.model.galaxies.light.bulge,
                hyper_galaxy=phase1.result.hyper_combined.instance.galaxies.light.hyper_galaxy,
            )
        ),
        hyper_background_noise=phase1.result.hyper_combined.instance.hyper_background_noise,
        real_space_mask=real_space_mask,
        search=search,
    )

    phase2.search.const_efficiency_mode = True
    phase2.search.n_live_points = 40
    phase2.search.facc = 0.8

    return ag.PipelineDataset(name, phase1, phase2)


if __name__ == "__main__":
    import sys

    runner.run(sys.modules[__name__])
